pub struct AxDashSet<T, S = BuildHasherDefault<AxHasher>>(/* private fields */);Expand description
Concurrent hash set backed by dashmap (multi-shard RwLock) with
AxHasher (AES-NI accelerated hashing) as the default hasher.
AxDashSet<T> is a thin newtype wrapper around DashSet<T> that adds
ergonomic ::new() / ::with_capacity() / ::with_shard_amount()
constructors. Every DashSet method is accessible transparently via
Deref.
Implementations
Sourceimpl<T> AxDashSet<T, BuildHasherDefault<AxHasher>>
impl<T> AxDashSet<T, BuildHasherDefault<AxHasher>>
Sourceimpl<T, S> AxDashSet<T, S>
impl<T, S> AxDashSet<T, S>
Sourcepub fn with_hasher(hasher: S) -> Self
pub fn with_hasher(hasher: S) -> Self
Creates an empty set using a custom BuildHasher.
Sourcepub fn with_capacity_and_hasher(capacity: usize, hasher: S) -> Self
pub fn with_capacity_and_hasher(capacity: usize, hasher: S) -> Self
Creates an empty set with at least capacity entries and a custom
BuildHasher.
Sourcepub fn into_inner(self) -> RawDashSet<T, S>
pub fn into_inner(self) -> RawDashSet<T, S>
Consumes the wrapper and returns the underlying RawDashSet.
Methods from Deref<Target = RawDashSet<T, S>>
Sourcepub fn hash_usize<T>(&self, item: &T) -> usizewhere
T: Hash,
pub fn hash_usize<T>(&self, item: &T) -> usizewhere
T: Hash,
Hash a given item to produce a usize. Uses the provided or default HashBuilder.
Sourcepub fn insert(&self, key: K) -> bool
pub fn insert(&self, key: K) -> bool
Inserts a key into the set. Returns true if the key was not already in the set.
Examples
use dashmap::DashSet;
let set = DashSet::new();
set.insert("I am the key!");Sourcepub fn remove<Q>(&self, key: &Q) -> Option<K>
pub fn remove<Q>(&self, key: &Q) -> Option<K>
Removes an entry from the map, returning the key if it existed in the map.
Examples
use dashmap::DashSet;
let soccer_team = DashSet::new();
soccer_team.insert("Jack");
assert_eq!(soccer_team.remove("Jack").unwrap(), "Jack");Sourcepub fn remove_if<Q>(&self, key: &Q, f: impl FnOnce(&K) -> bool) -> Option<K>
pub fn remove_if<Q>(&self, key: &Q, f: impl FnOnce(&K) -> bool) -> Option<K>
Removes an entry from the set, returning the key if the entry existed and the provided conditional function returned true.
use dashmap::DashSet;
let soccer_team = DashSet::new();
soccer_team.insert("Sam");
soccer_team.remove_if("Sam", |player| player.starts_with("Ja"));
assert!(soccer_team.contains("Sam"));use dashmap::DashSet;
let soccer_team = DashSet::new();
soccer_team.insert("Sam");
soccer_team.remove_if("Jacob", |player| player.starts_with("Ja"));
assert!(!soccer_team.contains("Jacob"));Sourcepub fn iter(&'a self) -> Iter<'a, K, S, DashMap<K, (), S>>
pub fn iter(&'a self) -> Iter<'a, K, S, DashMap<K, (), S>>
Creates an iterator over a DashMap yielding immutable references.
Examples
use dashmap::DashSet;
let words = DashSet::new();
words.insert("hello");
assert_eq!(words.iter().count(), 1);Sourcepub fn get<Q>(&'a self, key: &Q) -> Option<Ref<'a, K>>
pub fn get<Q>(&'a self, key: &Q) -> Option<Ref<'a, K>>
Get a reference to an entry in the set
Examples
use dashmap::DashSet;
let youtubers = DashSet::new();
youtubers.insert("Bosnian Bill");
assert_eq!(*youtubers.get("Bosnian Bill").unwrap(), "Bosnian Bill");Sourcepub fn shrink_to_fit(&self)
pub fn shrink_to_fit(&self)
Remove excess capacity to reduce memory usage.
Sourcepub fn retain(&self, f: impl FnMut(&K) -> bool)
pub fn retain(&self, f: impl FnMut(&K) -> bool)
Retain elements that whose predicates return true and discard elements whose predicates return false.
Examples
use dashmap::DashSet;
let people = DashSet::new();
people.insert("Albin");
people.insert("Jones");
people.insert("Charlie");
people.retain(|name| name.contains('i'));
assert_eq!(people.len(), 2);Sourcepub fn len(&self) -> usize
pub fn len(&self) -> usize
Fetches the total number of keys stored in the set.
Examples
use dashmap::DashSet;
let people = DashSet::new();
people.insert("Albin");
people.insert("Jones");
people.insert("Charlie");
assert_eq!(people.len(), 3);Sourcepub fn is_empty(&self) -> bool
pub fn is_empty(&self) -> bool
Checks if the set is empty or not.
Examples
use dashmap::DashSet;
let map = DashSet::<()>::new();
assert!(map.is_empty());Trait Implementations
Sourceimpl<T, S> Extend<T> for AxDashSet<T, S>
impl<T, S> Extend<T> for AxDashSet<T, S>
Sourcefn extend<I: IntoIterator<Item = T>>(&mut self, iter: I)
fn extend<I: IntoIterator<Item = T>>(&mut self, iter: I)
Sourcefn extend_one(&mut self, item: A)
fn extend_one(&mut self, item: A)
extend_one)Sourcefn extend_reserve(&mut self, additional: usize)
fn extend_reserve(&mut self, additional: usize)
extend_one)